using System;
using Microsoft.SPOT;
using System.Collections;

namespace Gadgeteer.Modules.Community.XBee
{
    /// <summary>
    /// Tests the ND AT command
    /// </summary>
    public class NodeDiscoveryTest : BaseTest
    {
        ArrayList eventArgs;

        /// <summary>
        /// Wire up the DiscoverNodesCompleted event handler
        /// </summary>
        /// <param name="context">TestContext</param>
        public override void Arrange(TestContext context)
        {
            context.XBee.DiscoverNodesCompleted += new DiscoverNodesCompletedEventHandler(XBee_DiscoverNodesCompleted);
            eventArgs = new ArrayList();
        }

        void XBee_DiscoverNodesCompleted(object sender, DiscoverNodesCompletedEventArgs e)
        {
            eventArgs.Add(e);
        }

        /// <summary>
        /// call DiscoverNodesAsync, which issues the ND AT command
        /// </summary>
        /// <param name="context">TestContext</param>
        public override void Act(TestContext context)
        {
            context.XBee.DiscoverNodesAsync();
        }

        /// <summary>
        /// Validate that no error occured, CommandStatus is 0x00, and record the NodeDescription
        /// </summary>
        /// <param name="context">TestContext</param>
        public override void Assert(TestContext context)
        {
            int numExpectedPeers = (int)context.Properties["NumExpectedPeers"];

            if (this.eventArgs.Count != numExpectedPeers)
            {
                throw new XBeeAssertFailed("Expected " + numExpectedPeers + " but DiscoverNodesCompleted only called " + eventArgs.Count + " times");
            }

            NodeDescription[] nodeDescription = new NodeDescription[this.eventArgs.Count];
            for (int i = 0; i < this.eventArgs.Count; i++)
            {
                DiscoverNodesCompletedEventArgs args = (DiscoverNodesCompletedEventArgs)this.eventArgs[i];
                if (args.Error != null)
                {
                    throw new XBeeAssertFailed(args.Error.ToString());
                }
                else if (args.Result.CommandStatus != 0x00)
                {
                    throw new XBeeAssertFailed("DiscoverNodes CommandStatus was not OK, it was " + HexUtil.ToHexString((byte)args.Result.CommandStatus));
                }
                else
                {
                    nodeDescription[i] = args.Result.NodeDescription;
                }
            }

            foreach (NodeDescription nd in nodeDescription)
            {
                Message += nd.ToString();
            }

            context.Peers = nodeDescription;
        }

        /// <summary>
        /// remove event handler
        /// </summary>
        /// <param name="context">TestContext</param>
        public override void Teardown(TestContext context)
        {
            context.XBee.DiscoverNodesCompleted -= XBee_DiscoverNodesCompleted;
        }
    }
}
